﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;

namespace PakExtract
{
    class Program
    {
        static void Main(string[] args)
        {
            var files = new string[] { "EOBDATA1.PAK", "EOBDATA2.PAK", "EOBDATA3.PAK", "EOBDATA4.PAK", "EOBDATA5.PAK" };
            var palettes = new List<FileInfo>();
            var allFiles = new List<FileInfo>();

            foreach (var pakFile in files)
            {
                Console.WriteLine(pakFile);
                var pak = PakFile.FromFile(pakFile);
                foreach (var file in pak.Files)
                {
                    allFiles.Add(file);

                    if (file.FileName.EndsWith(".COL"))
                    {
                        palettes.Add(file);
                        Console.WriteLine($"*PAL* {file.FileName,-12}\toffset: {file.OffsetInPak}\tsize: {file.FileSize} bytes");
                    }
                }
            }

            Console.WriteLine($"\n{palettes.Count} palette files found");
            var uniquePalettes = palettes.GroupBy(p => p.FileName).Select(g => g.First());

            if (!Directory.Exists("output"))
                Directory.CreateDirectory("output");
            
            foreach (var pakFile in files)
            {
                if (!Directory.Exists("output\\" + pakFile))
                    Directory.CreateDirectory("output\\" + pakFile);

                var pak = PakFile.FromFile(pakFile);
                foreach (var file in pak.Files)
                {
                    if (CpsFile.IsCps(file.RawData))
                    {
                        var image = CpsFile.FromData(file.RawData);
                        var imageName = file.FileName;

                        foreach (var pal in uniquePalettes)
                        {
                            var fileName = imageName.Replace(".", "_") + "_"
                                + pal.FileName.Replace(".", "_") + ".png";

                            Console.WriteLine($"Saving {fileName}");
                            SaveAsPng("output\\" + pakFile + "\\" + fileName, image.RawData, pal.RawData, true);
                        }
                    }
                }
            }

            Console.ReadKey();
        }

        public static void SaveAsPng(string filename, byte[] rawData, byte[] paletteData, bool transparent = false)
        {
            Bitmap bmp = new Bitmap(320, 200);
            var palette = new Color[paletteData.Length / 3];

            for (int i = 0; i < palette.Length; i++)
                palette[i] = Color.FromArgb(
                    (paletteData[i * 3] * 255) / 63, 
                    (paletteData[i * 3 + 1] * 255) / 63, 
                    (paletteData[i * 3 + 2] * 255) / 63);

            if (transparent)
                palette[0] = Color.Transparent;

            for (int i = 0; i < rawData.Length; i++)
                bmp.SetPixel(i % 320, i / 320, palette[rawData[i]]);

            bmp.Save(filename, ImageFormat.Png);
        }
    }
}
